## $Id: browser.py 11145 2014-02-21 06:28:51Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from zope.interface import Interface
from zope.component import getUtility, queryAdapter
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentRequestWebservicePageApplicant
    )
from waeup.kofa.browser.layout import KofaPage, UtilityView
from waeup.kofa.interfaces import IKofaUtils
from waeup.kofa.utils.helpers import to_timezone
from waeup.kwarapoly.students.interfaces import ICustomStudentOnlinePayment
from waeup.kwarapoly.applicants.interfaces import ICustomApplicantOnlinePayment
from waeup.kwarapoly.interfaces import MessageFactory as _

PRODUCT_ID = '3930'
SITE_NAME = 'kwarapoly.waeup.org'
PROVIDER_ACCT = '1010764827'
PROVIDER_BANK_ID = '117'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'KwaraPoly'
CURRENCY = '566'
GATEWAY_AMT = 300.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'

POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'

URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

class InterswitchPageStudent(KofaPage):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    grok.name('goto_interswitch')
    grok.template('student_goto_interswitch')
    grok.require('waeup.payStudent')
    label = _('Submit data to CollegePAY (Interswitch Payment Gateway)')
    submit_button = _('Submit')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = 'E6BA6CBBA9AF2871EE25C32C8D57C98895B9B001DC5B9CB2C463E2A9BDA44A3F1260C8A364F33789CDF74CB3EE7E6EF5D94F48D3AF7B727E75D97F07618DFA6D'

    def interswitch_img_url(self):
        return interswitch_img_url(self)

    def update(self):
        #if self.context.p_state != 'unpaid':
        if self.context.p_state == 'paid':
            self.flash(_("Payment ticket can't be re-send to CollegePAY."))
            self.redirect(self.url(self.context, '@@index'))
            return

        student = self.student = self.context.student
        certificate = getattr(student['studycourse'],'certificate',None)
        self.amount_auth = 100 * self.context.amount_auth
        xmldict = {}
        if certificate is not None:
            xmldict['department'] = certificate.__parent__.__parent__.code
            xmldict['faculty'] = certificate.__parent__.__parent__.__parent__.code
        else:
            xmldict['department'] = None
            xmldict['faculty'] = None
        self.category = getUtility(IKofaUtils).PAYMENT_CATEGORIES[self.context.p_category]
        tz = getUtility(IKofaUtils).tzinfo
        self.local_date_time = to_timezone(
            self.context.creation_date, tz).strftime("%Y-%m-%d %H:%M:%S %Z")
        self.site_redirect_url = self.url(self.context, 'request_webservice')
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = "0000000000000"
        xmldict['institution_bank_id'] = '0'
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 0.0
        dalash_amt = 200.0
        provider_amt = 300.0
        gateway_amt = GATEWAY_AMT
        self.pay_item_id = '000'
        if self.context.p_category == 'schoolfee':
            self.pay_item_id = '101'
            dalash_amt = 1800.0
            provider_amt = 1200.0
            if xmldict['faculty'] in ('CPGS',):
                xmldict['institution_acct'] = "1771180233"
                xmldict['institution_bank_id'] = '120'
            elif xmldict['faculty'] in ('IBAS',):
                xmldict['institution_acct'] = "2013910271"
                xmldict['institution_bank_id'] = '8'
                self.pay_item_id = '107'
            elif xmldict['faculty'] in ('IETS',):
                xmldict['institution_acct'] = "2013910271"
                xmldict['institution_bank_id'] = '8'
                self.pay_item_id = '109'
            elif xmldict['faculty'] in ('IFMS',):
                xmldict['institution_acct'] = "0106259811"
                xmldict['institution_bank_id'] = '10'
                self.pay_item_id = '106'
            elif xmldict['faculty'] in ('ITCH',):
                xmldict['institution_acct'] = "1771180233"
                xmldict['institution_bank_id'] = '120'
                self.pay_item_id = '110'
            elif xmldict['faculty'] in ('IICT',):
                xmldict['institution_acct'] = "2013910271"
                xmldict['institution_bank_id'] = '8'
                self.pay_item_id = '108'
        elif self.context.p_category == 'certificate':
            self.pay_item_id = '111'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'state_result':
            self.pay_item_id = '112'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'transcript_local':
            self.pay_item_id = '113'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'transcript_foreign':
            self.pay_item_id = '114'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'ver_result':
            self.pay_item_id = '115'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'change_course':
            self.pay_item_id = '116'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'change_inst':
            self.pay_item_id = '117'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'jamb_reject':
            self.pay_item_id = '118'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'cert_of_cert':
            self.pay_item_id = '119'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'ref_let':
            self.pay_item_id = '120'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'proc_cert':
            self.pay_item_id = '121'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'loss_idcard':
            self.pay_item_id = '122'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
            dalash_amt = 6.0
            xmldict['dalash_amt'] = 100 * dalash_amt
            provider_amt = 9.0
            gateway_amt = 15.0
        elif self.context.p_category == 'loss_examcard':
            self.pay_item_id = '123'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
            dalash_amt = 3.0
            xmldict['dalash_amt'] = 100 * dalash_amt
            provider_amt = 4.5
            gateway_amt = 7.5
        elif self.context.p_category == 'loss_result':
            self.pay_item_id = '124'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'loss_receipt':
            self.pay_item_id = '125'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'loss_clearance':
            self.pay_item_id = '126'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif self.context.p_category == 'conv_brochure':
            self.pay_item_id = '127'
            xmldict['institution_acct'] = "0131959715"
            xmldict['institution_bank_id'] = '10'
        elif 'maintenance' in self.context.p_category:
            self.pay_item_id = '102'
            xmldict['institution_acct'] = "0039050937"
            xmldict['institution_bank_id'] = '31'
            dalash_amt = 0.0
            provider_amt = 0.0


        xmldict['dalash_amt'] = 100 * dalash_amt
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt -
                gateway_amt - dalash_amt)


        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()

        # Interswitch amount is not part of the xml data

        if 'maintenance' in self.context.p_category:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="Dalash" item_amt="%(dalash_amt)d" bank_id="117" acct_num="1013196791" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = gateway_amt
        self.context.thirdparty_amt = dalash_amt
        return

class InterswitchPageApplicant(KofaPage):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    grok.require('waeup.payApplicant')
    grok.template('applicant_goto_interswitch')
    grok.name('goto_interswitch')
    label = _('Submit data to CollegePAY (Interswitch Payment Gateway)')
    submit_button = _('Submit')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '103'
    product_id = PRODUCT_ID
    mac = 'E6BA6CBBA9AF2871EE25C32C8D57C98895B9B001DC5B9CB2C463E2A9BDA44A3F1260C8A364F33789CDF74CB3EE7E6EF5D94F48D3AF7B727E75D97F07618DFA6D'

    def interswitch_img_url(self):
        return interswitch_img_url(self)

    def update(self):
        if self.context.p_state != 'unpaid':
            self.flash(_("Payment ticket can't be re-send to CollegePAY."))
            self.redirect(self.url(self.context, '@@index'))
            return
        if self.context.__parent__.__parent__.expired \
            and self.context.__parent__.__parent__.strict_deadline:
            self.flash(_("Payment ticket can't be send to CollegePAY. "
                         "Application period has expired."))
            self.redirect(self.url(self.context, '@@index'))
            return
        self.applicant = self.context.__parent__
        self.amount_auth = 100 * self.context.amount_auth
        xmldict = {}
        self.category = getUtility(
            IKofaUtils).PAYMENT_CATEGORIES[self.context.p_category]
        tz = getUtility(IKofaUtils).tzinfo
        self.local_date_time = to_timezone(
            self.context.creation_date, tz).strftime("%Y-%m-%d %H:%M:%S %Z")
        self.site_redirect_url = self.url(self.context, 'request_webservice')
        provider_amt = 300.0
        dalash_amt = 200.0
        if self.applicant.applicant_id.startswith('hnd'):
            xmldict['institution_acct'] = '7000016724'
            xmldict['institution_bank_id'] = '9'
        elif self.applicant.applicant_id.startswith('nd'):
            xmldict['institution_acct'] = '2014191363'
            xmldict['institution_bank_id'] = '8'
        elif self.applicant.applicant_id.startswith('pgft'):
            xmldict['institution_acct'] = '7000016724'
            xmldict['institution_bank_id'] = '9'
        elif self.applicant.applicant_id.startswith('prejambites'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
            self.pay_item_id = '104'
            provider_amt = 0.0
            dalash_amt = 0.0
        elif self.applicant.applicant_id.startswith('pre'):
            xmldict['institution_acct'] = '2013910271'
            xmldict['institution_bank_id'] = '8'
            provider_amt = 0.0
            dalash_amt = 0.0
        elif self.applicant.applicant_id.startswith('special'):
            if self.applicant.special_application == 'certificate':
                self.pay_item_id = '111'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'state_result':
                self.pay_item_id = '112'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'transcript_local':
                self.pay_item_id = '113'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'transcript_foreign':
                self.pay_item_id = '114'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'ver_result':
                self.pay_item_id = '115'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'change_course':
                self.pay_item_id = '116'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'change_inst':
                self.pay_item_id = '117'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'jamb_reject':
                self.pay_item_id = '118'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'cert_of_cert':
                self.pay_item_id = '119'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'ref_let':
                self.pay_item_id = '120'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'proc_cert':
                self.pay_item_id = '121'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'loss_idcard':
                self.pay_item_id = '122'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
                dalash_amt = 6.0
                xmldict['dalash_amt'] = 100 * dalash_amt
                provider_amt = 9.0
                gateway_amt = 15.0
            elif self.applicant.special_application == 'loss_examcard':
                self.pay_item_id = '123'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
                dalash_amt = 3.0
                xmldict['dalash_amt'] = 100 * dalash_amt
                provider_amt = 4.5
                gateway_amt = 7.5
            elif self.applicant.special_application == 'loss_result':
                self.pay_item_id = '124'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'loss_receipt':
                self.pay_item_id = '125'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'loss_clearance':
                self.pay_item_id = '126'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            elif self.applicant.special_application == 'conv_brochure':
                self.pay_item_id = '127'
                xmldict['institution_acct'] = "0131959715"
                xmldict['institution_bank_id'] = '10'
            else:
                xmldict['institution_acct'] = '00000000000'
                xmldict['institution_bank_id'] = '00'
        else:
            xmldict['institution_acct'] = '00000000000'
            xmldict['institution_bank_id'] = '00'
        xmldict['dalash_amt'] = 100 * dalash_amt
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_amt'] = 100 * (
            self.context.amount_auth - provider_amt - GATEWAY_AMT - dalash_amt)
        xmldict['institution_item_name'] = self.context.p_category
        xmldict['institution_name'] = INSTITUTION_NAME

        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()

        # Interswitch amount is not part of the xml data

        if not self.applicant.applicant_id.startswith('pre'):
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="Dalash" item_amt="%(dalash_amt)d" bank_id="117" acct_num="1013196791" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        self.context.thirdparty_amt = dalash_amt
        return

class InterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class InterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL